# DSpace directory structure

This is the directory structure 

```
├── clarin-dspace-docker
│   └── commul-customization
└── kubernetes
    ├── helm
    │   └── dspace
    │       ├── charts
    │       │   └── postgresql
    │       │       └── templates
    │       └── templates
    └── yaml
```


* `/clarin-dspace-docker` - Docker files, copied from https://github.com/commul/clarin-dspace-docker @ 74d06d3 . **With this directory, you can build images**
* `/kubernetes/helm` - Chart Directory. Refer to https://github.com/kubernetes/helm/blob/master/docs/charts.md#the-chart-file-structure :exclamation: **not working yet** :exclamation:
* `/kubernetes/yaml` - Useful for tests, but helm is a more flexible approach **With this directory, you can deploy images in kubernetes cluster**
# Introduction
# Prerequisites

* Kubernetes cluster

## To interact with cluster

* Kubernetes credentials
* a working [kubectl] (https://kubernetes.io/docs/tasks/tools/install-kubectl/)

## To interact with Helm server (tiller) -- Coming soon

* Kubernetes credentials
* a working [kubectl] (https://kubernetes.io/docs/tasks/tools/install-kubectl/)
* a working [helm client] (https://docs.helm.sh/using_helm/#quickstart-guide)

# How to use it
## Workflow
First of all, check if a docker image already exists.
If it exist, pull, otherwise, you need to build one.
If necessary create a [personal token](https://gitlab.scientificnet.org/profile/personal_access_tokens) and login into GitLab Registry.

```console
docker login gitlab.scientificnet.org:4567
```
 
### Use existing Docker image
```console
docker image ls -a gitlab.scientificnet.org:4567/commul/dspace/*
docker pull gitlab.scientificnet.org:4567/commul/dspace/<my_image>:<my tag>
```

### Build images
* build your docker image from `/clarin-dspace-docker` directory
* push to `https://gitlab.scientificnet.org/commul/dspace/container_registry` 

```console
docker build -t gitlab.scientificnet.org:4567/commul/dspace/<my_image>:<my tag> -f Dockerfile.XX .
docker push gitlab.scientificnet.org:4567/commul/dspace/<my_image>:<my tag>
```

# Installation
This step in a "workspace preparation" and is not directly related with dspace.

Provide gitlab & RBD credentials

```console
$ kubectl -n dspace-dev create -f gitlab-scientificnet-org-registry.yaml
```
```console
$ kubectl -n dspace-dev create -f rbd-secret.yaml
```
## PostgreSQL
To install PostgreSQL, we need some passwords, so we need Kubernetes secrets

```console
$ kubectl -n dspace-dev create -f postgresql-secret.yaml 
secret "dspace-postgresql" created
```

```console
$ kubectl -n dspace-dev create -f dspace-secrets.yaml 
secret "dspace-secrets" created
```

```console
$ kubectl -n dspace-dev get secrets
NAME                                TYPE                                  DATA      AGE
ceph-rbd-secret                     kubernetes.io/rbd                     1         14m
default-token-c1pd4                 kubernetes.io/service-account-token   3         4d
dspace-postgresql                   Opaque                                1         1h
dspace-secrets                      Opaque                                1         1m
gitlab-scientificnet-org-registry   kubernetes.io/dockerconfigjson        1         46m
```



Next, create a RBD Claim
```console
$ kubectl -n dspace-dev create -f postgresql-pvc.yaml 
persistentvolumeclaim "dspace-postgresql" created

$ kubectl -n dspace-dev get pvc
NAME                STATUS    VOLUME                                     CAPACITY   ACCESSMODES   STORAGECLASS   AGE
dspace-postgresql   Bound     pvc-610c358d-a518-11e7-b21c-b60483de6a40   8Gi        RWO           auto           2s
```

Ok, now we are ready to deploy our PostgreSQL!
```console
$ kubectl -n dspace-dev create -f postgresql.yaml 

$ kubectl -n dspace-dev get po
NAME                                 READY     STATUS              RESTARTS   AGE
dspace-postgresql-1806947503-cj077   0/1       ContainerCreating   0          2s

$ kubectl -n dspace-dev get po
NAME                                 READY     STATUS    RESTARTS   AGE
dspace-postgresql-1806947503-qw8q4   0/1       Running   0          9s

$ kubectl -n dspace-dev get po
NAME                                 READY     STATUS    RESTARTS   AGE
dspace-postgresql-1806947503-qw8q4   1/1       Running   0          46s
```

We have now launched the pod but we have to wait until the pod is up before accessing it
```console
$ kubectl -n dspace-dev exec -it dspace-postgresql-1806947503-qw8q4 bash
root@dspace-postgresql-1806947503-qw8q4:/#
```

Also, we can also test our PostgreSQL installation with a port forward
```console
$ kubectl -n dspace-dev port-forward dspace-postgresql-1806947503-qw8q4 5432:5432
```
now we are able to connect to localhost:5432
![pgAdmin_connect](pgadmin_1.png)
![pgAdmin_showTables](pgadmin_2.png)

```console
$ kubectl -n dspace-dev create -f postgresql-service.yaml
service "dspace-postgresql" created
```
```console
$ kubectl -n dspace-dev exec -it servlet env | grep POSTGRESQL
DSPACE_POSTGRESQL_SERVICE_PORT_POSTGRESQL=5432
DSPACE_POSTGRESQL_PORT_5432_TCP_ADDR=10.13.0.173
DSPACE_POSTGRESQL_SERVICE_HOST=10.13.0.173
DSPACE_POSTGRESQL_PORT_5432_TCP_PROTO=tcp
DSPACE_POSTGRESQL_PORT_5432_TCP_PORT=5432
DSPACE_POSTGRESQL_SERVICE_PORT=5432
DSPACE_POSTGRESQL_PORT=tcp://10.13.0.173:5432
DSPACE_POSTGRESQL_PORT_5432_TCP=tcp://10.13.0.173:5432
```

## DSpace
```console
$ kubectl -n dspace-dev create -f servlet.yaml 
pod "servlet" created
```
```console
$ kubectl -n dspace-dev get po
NAME                                 READY     STATUS    RESTARTS   AGE
dspace-postgresql-1806947503-qw8q4   1/1       Running   0          4d
servlet                              1/1       Running   0          34s
```
```console
$ kubectl -n dspace-dev logs servlet
[Wed Oct  4 12:24:25 UTC 2017] Starting 
[Wed Oct  4 12:24:25 UTC 2017] Cloning lindat-dspace repository...skip! /opt/repository/sources/dspace already exists 
[Wed Oct  4 12:24:25 UTC 2017] Creating workspace...skip! /opt/repository/workspace already exists 
[Wed Oct  4 12:24:25 UTC 2017] Copy makefile...skip! /opt/repository/workspace/config/variable.makefile already exists
[Wed Oct  4 12:24:25 UTC 2017] Copy local.properties...skip! /opt/repository/workspace/sources/local.properties already exists
[Wed Oct  4 12:24:25 UTC 2017] Compilation and Deployment of DSpace...already done 
already done 
already done 
[Wed Oct  4 12:24:25 UTC 2017] Copy aai_config.js...skip! /opt/lindat-dspace/installation/webapps/xmlui/themes/UFAL/lib/js/aai_config.js already exists
[Wed Oct  4 12:24:25 UTC 2017] Create dspace admin...already done 
[Wed Oct  4 12:24:25 UTC 2017] Starting Tomcat...Starting Tomcat 8...
Tomcat started.
```
```console
$ kubectl -n dspace-dev port-forward servlet 8080:8080
Forwarding from 127.0.0.1:8080 -> 8080
Forwarding from [::1]:8080 -> 8080
```

connect to http://localhost:8080/repository/xmlui/#
